select 'Removing all temporary tables if they exist';
drop table if exists RL_temp;
drop table if exists CRS_temp;
drop table if exists sums;
drop table if exists counts;

set @totalRows = 
(
	select count(*)
	from files
);

select 'Creating percentage frequency per library';
create table RL_temp
    select toFileId, CAST(count(*) / @totalRows AS DECIMAL(10,9)) as RL
    from deps
    group by toFileId;

select 'Updating files with RL score';
update files f
inner join RL_temp t
on f.fileId = t.toFileId
set f.RL = t.RL;

select 'Creating counts and sums for CRS';
create table counts
    select fromFileId, count(*) * count(*) as square
    from deps
    group by fromFileId;

create table sums
    select fromFileId, sum(RL) as sommetje
    from
    (
        select d.fromFileId, d.toFileId, f.RL
        from deps d
        inner join files f
        on d.toFileId = f.fileId
    ) a
    group by fromFileId;

select 'Creating indices for fast lookups';
create index fileIdIDX 
	on counts (fromFileId) 
	using hash;
	
create index fileIdIDX 
    on sums (fromFileId) 
    using hash;

select 'Creating temporary CRS table';
create table CRS_temp
    select c.fromFileId, s.sommetje / c.square as CRS
    from counts c
    inner join sums s
    on c.fromFileId = s.fromFileId;

select 'Updating files table with final CRS score';
update files
inner join CRS_temp crs
on files.fileId = crs.fromFileId
set files.CRS = crs.CRS;

select 'Removing temporary tables';
drop table RL_temp;
drop table CRS_temp;
drop table sums;
drop table counts;