USE `` ;

-- -----------------------------------------------------
-- Table ``.`files`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`files` (
  `fileId` INT NOT NULL AUTO_INCREMENT ,
  `fullName` VARCHAR(255) NULL ,
  `groupId` VARCHAR(50) NOT NULL ,
  `artifactId` VARCHAR(50) NOT NULL ,
  `version` VARCHAR(50) NOT NULL ,
  `reservedNodeId` INT NOT NULL ,
  `snapshotId` INT NULL ,
  PRIMARY KEY (`fileId`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`unitTypes`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`unitTypes` (
  `unitTypeId` INT NOT NULL ,
  `parentType` INT NULL ,
  `description` VARCHAR(255) NOT NULL ,
  PRIMARY KEY (`unitTypeId`) ,
  INDEX `parentTypeFK` (`parentType` ASC) ,
  CONSTRAINT `parentTypeFK`
    FOREIGN KEY (`parentType` )
    REFERENCES ``.`unitTypes` (`unitTypeId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`units`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`units` (
  `unitId` INT NOT NULL AUTO_INCREMENT ,
  `name` VARCHAR(255) NULL ,
  `unitType` INT NOT NULL ,
  `parentId` INT NULL ,
  `fileId` INT NOT NULL ,
  `LOC` INT NULL ,
  `McCabe` INT NULL ,
  `Params` INT NULL ,
  PRIMARY KEY (`unitId`) ,
  INDEX `fileIdFK` (`fileId` ASC) ,
  INDEX `parentIdFK` (`parentId` ASC) ,
  INDEX `unitTypeFK` (`unitType` ASC) ,
  CONSTRAINT `fileIdFK`
    FOREIGN KEY (`fileId` )
    REFERENCES ``.`files` (`fileId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `parentIdFK`
    FOREIGN KEY (`parentId` )
    REFERENCES ``.`units` (`unitId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `unitTypeFK`
    FOREIGN KEY (`unitType` )
    REFERENCES ``.`unitTypes` (`unitTypeId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`changeTypes`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`changeTypes` (
  `changeTypeId` INT NOT NULL ,
  `description` VARCHAR(255) NOT NULL ,
  `breaking` TINYINT(1) NOT NULL ,
  PRIMARY KEY (`changeTypeId`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`changes`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`changes` (
  `changeId` INT NOT NULL AUTO_INCREMENT ,
  `changeType` INT NOT NULL ,
  `fromUnit` INT NULL ,
  `toUnit` INT NULL ,
  PRIMARY KEY (`changeId`) ,
  INDEX `fromUnitFK` (`fromUnit` ASC) ,
  INDEX `toUnitFK` (`toUnit` ASC) ,
  INDEX `changeTypeFK` (`changeType` ASC) ,
  CONSTRAINT `fromUnitFK`
    FOREIGN KEY (`fromUnit` )
    REFERENCES ``.`units` (`unitId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `toUnitFK`
    FOREIGN KEY (`toUnit` )
    REFERENCES ``.`units` (`unitId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `changeTypeFK`
    FOREIGN KEY (`changeType` )
    REFERENCES ``.`changeTypes` (`changeTypeId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`callTypes`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`callTypes` (
  `callTypeId` INT NOT NULL ,
  `description` VARCHAR(255) NOT NULL ,
  PRIMARY KEY (`callTypeId`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`calls`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`calls` (
  `callId` INT NOT NULL AUTO_INCREMENT ,
  `fromUnit` INT NOT NULL ,
  `toUnit` INT NOT NULL ,
  `callType` INT NOT NULL ,
  PRIMARY KEY (`callId`) ,
  INDEX `callTypeFK` (`callType` ASC) ,
  INDEX `fromUnitFK2` (`fromUnit` ASC) ,
  INDEX `toUnitFK2` (`toUnit` ASC) ,
  CONSTRAINT `callTypeFK`
    FOREIGN KEY (`callType` )
    REFERENCES ``.`callTypes` (`callTypeId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fromUnitFK2`
    FOREIGN KEY (`fromUnit` )
    REFERENCES ``.`units` (`unitId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `toUnitFK2`
    FOREIGN KEY (`toUnit` )
    REFERENCES ``.`units` (`unitId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`runs`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`runs` (
  `runId` INT NOT NULL AUTO_INCREMENT ,
  `programId` INT NOT NULL ,
  `nodeId` INT NOT NULL ,
  `startTime` TIMESTAMP NOT NULL ,
  `endTime` TIMESTAMP NULL ,
  `status` INT NOT NULL ,
  PRIMARY KEY (`runId`) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`errors`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`errors` (
  `errorId` INT NOT NULL AUTO_INCREMENT ,
  `runId` INT NOT NULL ,
  `message` VARCHAR(255) NOT NULL ,
  `stackTrace` VARCHAR(1000) NOT NULL ,
  PRIMARY KEY (`errorId`) ,
  INDEX `runIdFK` (`runId` ASC) ,
  CONSTRAINT `runIdFK`
    FOREIGN KEY (`runId` )
    REFERENCES ``.`runs` (`runId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table ``.`fileruns`
-- -----------------------------------------------------
CREATE  TABLE IF NOT EXISTS ``.`fileruns` (
  `runId` INT NOT NULL ,
  `fileId` INT NOT NULL ,
  PRIMARY KEY (`runId`, `fileId`) ,
  INDEX `runIdFKIDX` (`runId` ASC) ,
  INDEX `fileIdFKIDX` (`fileId` ASC) ,
  CONSTRAINT `runIdFK2`
    FOREIGN KEY (`runId` )
    REFERENCES ``.`runs` (`runId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fileIdFK2`
    FOREIGN KEY (`fileId` )
    REFERENCES ``.`files` (`fileId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- procedure clearTables
-- -----------------------------------------------------

DELIMITER $$
USE ``$$
CREATE DEFINER=``@`` PROCEDURE `clearTables`()
BEGIN
    set foreign_key_checks = 0;

    delete from errors;
    delete from runs;
    delete from fileruns;
    delete from files;
    delete from changes;
    delete from calls;
    delete from units;
    
    alter table errors auto_increment = 1;
    alter table runs auto_increment = 1;
    alter table fileruns auto_increment = 1;
    alter table files auto_increment = 1;
    alter table changes auto_increment = 1;
    alter table calls auto_increment = 1;
    alter table units auto_increment = 1;

    set foreign_key_checks = 1;
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure endRun
-- -----------------------------------------------------

DELIMITER $$
USE ``$$
CREATE DEFINER=``@`` PROCEDURE `endRun`(runIdParam INTEGER, endTimeParam TIMESTAMP, completelyFinished BOOLEAN)
BEGIN
    set @errorCount = 
    (
        select count(*)
        from errors
        where runId = runIdParam
    );

    set @status = 0;

    if (completelyFinished) then
        if (@errorCount = 0) then
            set @status = 1;
        else
            set @status = 2;
        end if;
    else
        set @status = 3;
    end if;

    -- Update the run with a finishing time
    update runs
    set endTime = endTimeParam,
        status = @status
    where runId = runIdParam;
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure fillTypeTables
-- -----------------------------------------------------

DELIMITER $$
USE ``$$
CREATE DEFINER=``@`` PROCEDURE `fillTypeTables`()
BEGIN
    -- callTypes
    insert into callTypes (callTypeId, description)
        values (1, 'Dependency between JARs as specified in a Maven pom.xml file');
    insert into callTypes (callTypeId, description)
        values (2, 'Method call');

    -- changeTypes
    insert into changeTypes (changeTypeId, description, breaking)
        values (1, 'Add API method, if method need not be implemented by client', false);
    insert into changeTypes (changeTypeId, description, breaking)
        values (2, 'Add API method, if method must be implemented by client', true);
    insert into changeTypes (changeTypeId, description, breaking)
        values (3, 'Delete API method', true);

    -- unitTypes
    insert into unitTypes (unitTypeId, parentType, description)
        values (1, NULL, 'JAR file');
    insert into unitTypes (unitTypeId, parentType, description)
        values (2, 1, 'Package');
    insert into unitTypes (unitTypeId, parentType, description)
        values (3, 2, 'Class');
    insert into unitTypes (unitTypeId, parentType, description)
        values (4, 3, 'Method');

    
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure insertDependency
-- -----------------------------------------------------

DELIMITER $$
USE ``$$


CREATE DEFINER=``@`` PROCEDURE `insertDependency`(fileIdParam INTEGER, groupIdParam VARCHAR(50), artifactIdParam VARCHAR(50), versionParam VARCHAR(50))
BEGIN
    set @firstUnitId = 
    (
        select unitId
        from units u
        where u.fileId = fileIdParam
        limit 1
    );

    set @secondUnitId = 
    (
        select u.unitId
        from units u
        inner join files f
            on f.fileId = u.fileId
        where f.groupId = groupIdParam 
            and f.artifactId = artifactIdParam
            and f.version = versionParam
        limit 1
    );

    if (@secondUnitId is not null) then
        select 'Matching JAR file found.' union
        select CONCAT('firstUnitId: ', @firstUnitId, ', secondUnitId: ', @secondUnitId);

        -- Found second file id, so jar file exists on disk
        -- callType 1 is dependency between JAR files
        insert into calls (fromUnit, toUnit, callType)
             values (@firstUnitId, @secondUnitId, 1);
    else
        select 'No matching JAR file found, creating new one' union
        select CONCAT('groupId: ', groupIdParam,', artifactId: ', artifactIdParam,', version: ',versionParam);

        -- Not found, no matching dependency on disk. Make a new one without filename
        insert into files (fullName, groupId, artifactId, version)
            values (null, groupIdParam, artifactIdParam, versionParam);
        insert into units (unitType, parentId, fileId, LOC, McCabe, Params)
            values (1, NULL, LAST_INSERT_ID(), NULL, NULL, NULL);

        select CONCAT('Creating call with link between', @firstUnitId, ' and ', LAST_INSERT_ID());

        -- Link the just created ID with this fileId
        insert into calls (fromUnit, toUnit, callType)
            values (@firstUnitId, LAST_INSERT_ID(), 1);
    end if;
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure insertFile
-- -----------------------------------------------------

DELIMITER $$
USE ``$$
CREATE DEFINER=``@`` PROCEDURE `insertFile`(fullNameParam VARCHAR(255), groupIdParam VARCHAR(50), artifactIdParam VARCHAR(50), 
                                    versionParam VARCHAR(50), reservedNodeIdParam INTEGER, snapshotIdParam INTEGER)
BEGIN
    insert into files (fullName, groupId, artifactId, version, reservedNodeId, snapshotId)
        values (fullNameParam, groupIdParam, artifactIdParam, versionParam, reservedNodeIdParam, snapshotIdParam);

    -- unitType 1 is JAR file, which does not have a parent
    -- For JAR files, the LOC is not calculated separately but assumed 
    -- to be equal to the sum of all packages/classes/methods
    -- Calculating the LOC and the number of methods, fields, classes on higher level
    -- than methods can be done later using aggregation with group by queries. 
    insert into units (unitType, parentId, fileId, LOC, McCabe, Params)
        values (1, NULL, LAST_INSERT_ID(), NULL, NULL, NULL);
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure insertMethod
-- -----------------------------------------------------

DELIMITER $$
USE ``$$


CREATE DEFINER=``@`` PROCEDURE `insertMethod`(methodNameParam VARCHAR(255), fileIdParam INTEGER, locParam INTEGER, 
mccabeParam INTEGER, nrParamsParam INTEGER, classNameParam VARCHAR(255), packageNameParam VARCHAR(255))
BEGIN
    -- THis is the parent of each package, it always exists.
    set @fileUnitId = 
    (
        select unitId
        from units
        where fileId = fileIdParam
        limit 1
    );
    
    -- Create or get package
    set @packageId = 
    (
        select unitId
        from units
        where name = packageNameParam
            and fileId = fileIdParam
        limit 1
    );

    if (@packageId is null) then
        -- Package does not exist yet, create it
        insert into units (name, unitType, parentId, fileId, LOC, McCabe, Params)
            values (packageNameParam, 2, @fileUnitId, fileIdParam, null, null, null);        

        set @packageId = LAST_INSERT_ID();
    end if;

    -- Create or get class
    set @classId = 
    (
        select unitId
        from units
        where name = classNameParam
            and fileId = fileIdParam
    );

    if (@classId is null) then
        insert into units (name, unitType, parentId, fileId, LOC, McCabe, Params)
            values (classNameParam, 3, @packageId, fileIdParam, null, null, null);
    
        set @classId = LAST_INSERT_ID();
    end if;


    -- Create method. It cannot exist already
    insert units (name, unitType, parentId, fileId, LOC, McCabe, Params) 
        values (methodNameParam, 4, @classId, fileIdParam, locParam, mccabeParam, nrParamsParam);
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure log
-- -----------------------------------------------------

DELIMITER $$
USE ``$$


CREATE DEFINER=``@`` PROCEDURE `log`(runIdParam INTEGER, messageParam VARCHAR(255), stackTraceParam VARCHAR(1000))
BEGIN
    insert into errors(runId, message, stackTrace)
        values (runIdParam, messageParam, stackTraceParam);
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure selectBatch
-- -----------------------------------------------------

DELIMITER $$
USE ``$$


CREATE DEFINER=``@`` PROCEDURE `selectBatch`(nodeIdParam INTEGER)
BEGIN
    select *
    from files f
    where reservedNodeId = nodeIdParam
        or nodeIdParam = -1 -- For debugging purposes, -1 == ignore nodes
    order by f.fileId;
END$$

DELIMITER ;

-- -----------------------------------------------------
-- procedure showDependencies
-- -----------------------------------------------------

DELIMITER $$
USE ``$$


CREATE DEFINER=``@`` PROCEDURE `showDependencies`()
BEGIN
    select 
        f1.groupId as 'fromGroupId',
        f1.artifactId as 'fromArtifactId',
        f1.version as 'fromVersion',
        f2.groupId as 'toGroupId',
        f2.artifactId as 'toArtifactId',
        f2.version as 'toVersion'
    from files f1
    inner join units u1
        on u1.fileId = f1.fileId
    inner join calls c
        on c.fromUnit = u1.unitId
    inner join units u2
        on c.toUnit = u2.unitId
    inner join files f2
        on u2.fileId = f2.fileId;
END$$

DELIMITER ;

-- -----------------------------------------------------
-- function startRun
-- -----------------------------------------------------

DELIMITER $$
USE ``$$
CREATE DEFINER=``@`` FUNCTION `startRun`(programIdParam INTEGER, 
    nodeIdParam INTEGER, startTimeParam TIMESTAMP) RETURNS int(11)
BEGIN
    insert into runs(programId, nodeId, startTime, status)
        values (programIdParam, nodeIdParam, startTimeParam, 0);
    
    set @runId = LAST_INSERT_ID();
    
    -- programId 0 is filescan, does not enter rows in fileruns
    if (programIdParam != 0) then
        -- Update those  fileIds in fileruns
        insert into fileruns(runId, fileId)
            select @runId, f.fileId
            from files f
            where f.reservedNodeId = nodeIdParam
                or nodeIdParam = -1
            order by f.fileId;
    end if;

    RETURN @runId;
END$$

DELIMITER ;
