#!/bin/bash

usage() {
cat << EOF
This script retrieves method, class, package and jar file information from
the Berkeley DB database.

NOTE: When using special characters in your query, make sure to escape them!
Also, remove spaces between arguments.

usage: $0 options

OPTIONS:
  -h	Show this message
  -c	Print output in CSV format
  -j	The directory of the executable jar file
  -b	The directory of the Berkeley DB database
  -g	The groupId to search for
  -a	The artifactId to search for
  -v	The version to search for
  -n	The Fully Qualified Name to search for
  -f	The fileId to search for
  -s	The snapshotId to search for
  -u	The unitType to search for 
	(1 = jar file, 2 = package, 3 = java file, 4 = class, 5 = method)
  -i	The unitId to search for
  
Example: ./getunits.sh -b berkeley/units -j ../..//jar/.jar 
   -n "org.reflext.core.TypeInfoImpl\\\$constructor(org.reflext.core.TypeDomain,argument2)"
 
EOF
}

csv=
jarFile=
berkeleyDb=
groupId=
artifactId=
version=
fqn=
fileId=
snapshotId=
unitType=
unitId=

while getopts "hcj:b:g:a:v:n:f:s:u:i:" OPTION
do
	case $OPTION in
		h)
			usage
			exit 1
			;;
		c)
			csv="-csv"
			;;
		j)
			jarFile=$OPTARG
			;;
		b)
			berkeleyDb="-unitDb "$OPTARG
			;;
		g)
			groupId="-groupId "$OPTARG
			;;
		a)
			artifactId="-artifactId "$OPTARG
			;;
		v)
			version="-version "$OPTARG
			;;
		n)
			fqn="-fqn \""$OPTARG"\""
			;;
		f)
			fileId="-fileId "$OPTARG
			;;
		s)
			snapshotId="-snapshotId "$OPTARG
			;;
		u)
			unitType="-unitType "$OPTARG
			;;
		i)
			unitId="-unitId "$OPTARG
			;;
	esac
done

if [[ $jarFile == "" ]]; then
	echo ".jar location must be supplied"
	exit 1
fi

if [[ $berkeleyDb == "" ]]; then
	echo "Berkeley DB database location must be supplied"
	exit 1
fi

java -Xmx2g -jar $jarFile -s -ExtractBerkeleyDbData $berkeleyDb $csv $unitId $groupId $artifactId $version $fqn $fileId $snapshotId $unitType $unitId