#!/usr/bin/env python

import os
import sys
import pickle
import MySQLdb as db
import networkx as nx

def main():
	G = load_from()
	
	(pagerank, hubbiness, authoritativeness) = calculate_metrics(G)
	save_metric_to("PageRank", pagerank)
	save_metric_to("Hubbiness", hubbiness)
	save_metric_to("Authoritativeness", authoritativeness)
	
	print("Finished.")

# ==============================
def load_from():
	conn = connect()
	cursor = get_dependencies(conn)
	rows = cursor.fetchall()
	G = nx.DiGraph()
	for row in rows:
		process_row(row, G)
	cursor.close()
	conn.close()
	return G

def connect():
	print("Connecting to database...")
	conn = db.connect(host = "",
					   = "",
					  passwd = "",
					  db = "")
	return conn

def get_dependencies(conn):
	print("Fetching dependencies...")
	cursor = conn.cursor()
	cursor.execute("select fromFileId, toFileId from deps")
	return cursor

def process_row(row, G):
	node1FileId = row[0]
	node2FileId = row[1]
	G.add_edge(node1FileId, node2FileId)

def calculate_metrics(G):
	print("Number of nodes in graph: " + str(G.number_of_nodes()))
	print("Number of edges in graph: " + str(G.number_of_edges()))
	pagerank = calculate_metric("pagerank", G)
	(hubbiness, authoritativeness) = calculate_metric("hits", G)
	
	return (pagerank, hubbiness, authoritativeness)
	
def calculate_metric(metricname, G):
	print("Calculating " + metricname + "...")
	metricdict = None
	
	if metricname == "pagerank":
		metricdict = nx.pagerank(G, alpha=0.9)
	elif metricname == "betweenness":
		metricdict = nx.betweenness_centrality(G)
	elif metricname == "hits":
		metricdict = nx.hits(G)		
	print("Finished calculating " + metricname)
	
	return metricdict

def save_metric_to(metricname, metriclist):
	conn = connect()
	cursor = conn.cursor()
	
	count = 0
	
	for (nodeId, metricvalue) in metriclist.items():
		count = count + 1
		
		if count % 1000 == 0:
			print("select 'updated {0} rows'".format(str(count)))
		
		print("UPDATE ``.files SET {0} = {1} WHERE fileId = {2}".format(metricname, "{0:.20f}".format(metricvalue), str(nodeId)))
	
	close(cursor, conn)

def close(cursor, conn):
	print("Closing connections...")
	cursor.close()
	conn.close()

main()
