#!/bin/bash

if [ $# -lt 1 ]; then
	echo "Usage: $0 <fullName>" >&2
	exit 1
fi

# 1st argument: the full name of the base directory to run the SAT on
baseDir=$1
scriptsDir=$(pwd)

=
=
programDir=
host=

baseDir=$(cd $baseDir; pwd)
nodeId=$(hostname | sed 's:\.:_:g' | sed 's:-:_:g')
echo "Using $nodeId"
tempDir=

echo "Cleaning temporary directory"
rm -rf $tempDir/* 2>/dev/null
mkdir -p $tempDir

echo "=== STARTING runsample.sh ==="
cd $baseDir
echo "*** Investigating $(pwd)"

query="select CONCAT(version,';',satDate) from files where fullName like '$(pwd)%' and enabled = 1 and hasSource = 1"
echo "Executing query \"$query\""

if [ $host == "" ]; then
	allVersions=$(mysql -u$ -p$ -D -B -N -e "$query")
else
	allVersions=$(ssh $host "mysql -u$ -p$ -D -B -N -e \"$query\"")
fi

# Iterate over all subdirectories, which are versions of the library
for versionAndDate in $allVersions; do
	echo "Retrieved line '$versionAndDate' from getversions.sh"
	dir=$(echo $versionAndDate | cut -d';' -f1)
	satDate=$(echo $versionAndDate | cut -d';' -f2 | sed 's:-::g')
	
	if [ -z $satDate ]; then
		echo "runsample.sh: missing satDate. Quitting now."
		exit 1
	fi
	if [ -z $dir ]; then
		echo "runsample.sh: missing directory. Quitting now."
		exit 1
	fi
	
	echo "satDate: '$satDate'"
	cd $dir
	echo " => looking into '$(pwd)'"
	finalOutputDir="$tempDir/$nodeId-$satDate"
	echo "finalOutputDir: '$finalOutputDir'"
	
	sourceJar=$(find $(pwd) -type f -name "*.jar" -maxdepth 1 | grep ".*-sources.jar$")
	
	if [ -z $sourceJar ]; then
		echo "No source jar found in '$(pwd)', continuing."
	else
		mkdir -p $finalOutputDir
		$scriptsDir/unzipsources.sh $sourceJar $finalOutputDir
		if [ $? -ne 0 ]; then
			echo "Extraction failed. Removing files."
			cd $baseDir
			rm -rf $finalOutputDir
			continue
		fi
	fi
	
	echo ""
	cd $baseDir
done

if [ -z $tempDir ]; then
	echo "Directory string '$tempDir' is empty. Exiting now."
	exit 1
fi

cd $scriptsDir

if [ $(echo $(ls -1 | wc -l)) -eq 0 ]; then
	echo "No snapshots found. Not running SAT."
	cd $scriptsDir
	exit 1
else
	echo "Running SAT on '$tempDir'"
	$scriptsDir/runsat.sh $tempDir system
	cd $scriptsDir
	exit 0
fi
