#!/bin/bash

standardScope=

scriptDir=$(pwd)
cd $scriptDir
# Input parameter 1: the base directory to run the SAT on
baseDir=$(cd $1; pwd)
depth=1
nodeId=$(hostname | sed 's:\.:_:g' | sed 's:-:_:g')

tmpXmlScopeDir=

if [ ! -d $tmpXmlScopeDir ]; then
	echo "Temporary scope directory '$tmpXmlScopeDir' does not exist. Exiting now."
	exit 1
fi

tmpXmlScopeFile=$tmpXmlScopeDir/scope.xml

dirName=$(echo $baseDir | sed 's:\/\.::g' | sed 's:/:\\/:g')
scopeFile=$(echo $standardScope | sed 's/<DIR>/'$dirName'/' | sed 's/<NODENAME>/'$nodeId'/')
echo $scopeFile > $tmpXmlScopeFile

if [ -e $tmpXmlScopeFile ]; then
	echo "Temporary scope file '$tmpXmlScopeFile' successfully created."
else
	echo "Problem in creating the temporary scope file '$tmpXmlScopeFile'. Exiting now."
	exit 1
fi

echo "-------"
echo "Running SAT with dirName: $baseDir. Writing scope file to $tmpXmlScopeFile"
echo "scopeFile: $scopeFile"
echo "-------"
echo "Current working directory: $(pwd)"

pythonVersion=$(python -V 2>&1)
echo "* Python version loaded: $pythonVersion"
javaVersion=$(java -showversion 2>&1); javaVersion=$(echo $javaVersion | cut -c1-140)
echo "* Java version loaded: $javaVersion"

echo "runsat.sh: Connecting from a node"
logfileDir=
sat=
host=

mkdir -p $logFileDir

./datarunner.sh $tmpXmlScopeFile $baseDir $host $logFileDir/runner
./dataimporter.sh $tmpXmlScopeFile $baseDir $host $logFileDir/importer
#$sat run-scope $tmpXmlScopeFile -host  -logdir $logFileDir

echo "Done running SAT."
cd $scriptDir