#!/usr/bin/env Rscript

# Create different scatterplots for paper
cat("Loading file from hypotheses.r…\n")
load(file=".RData")

png("../paper/scatterMaintUnits.png", height=1000, width=1000, 
	units="px", points=30, antialias="subpixel") 

cat("Creating a scatterplot for maintainability vs log(nrUnits)…\n")
plot(log(dataset$nrUnits), (dataset$maintainability/dataset$vol), cex=0.3, 
	col=rgb(160,160,160,50,maxColorValue=255), pch=16, 
	main="",
	xaxt="n", yaxt="n",
	xlim=c(0,10), ylim=c(0.3, 0.8), 
	xlab="", ylab="")
axis(side=1, at=c(0.04, seq(1, 10, by=1)), labels=seq(0, 10, by=1), padj=-1.0)
axis(side=2, at=seq(0.3, 0.8, by=0.1), labels=seq(0.3, 0.8, by=0.1), padj=1.0)
abline(lm((dataset$maintainability/dataset$vol) ~ log(dataset$nrUnits)))
label <- expression(paste(rho, " = -0.51"))
text(8.7, 0.5, labels=label)
nrPoints <- nrow(dataset[!is.na(dataset$nrUnits) & !is.na(dataset$maintainability),])
text(9, 0.3, labels=paste("n = ",nrPoints,sep=""), font=3)
title(xlab="log(nrUnits)", line=1.2)
title(ylab="Maintainability / volume", line=1.5)

dev.off()

png("../paper/scatterMaintCEM.png", height=1000, width=1000, 
	units="px", points=30, antialias="subpixel") 

dataset$logCEM <- log(dataset$CEM)
dataset[dataset$logCEM == -Inf & !is.na(dataset$logCEM),]$logCEM <- 0
dataset$logCEM <- (-1 * dataset$logCEM)
cat("Creating a scatterplot for maintainability vs log(CEM)…\n")
plot(dataset$logCEM, dataset$maintainability, cex=0.3, 
	col=rgb(100,100,100,50,maxColorValue=255), pch=16, 
	main="", 
	xaxt="n", yaxt="n",
	ylim=c(1.5, 4.5), xlim=c(0.5, 12),
	xlab="", ylab="")
axis(side=1, at=c(0.04, seq(2, 12, by=2)), labels=seq(0, 12, by=2), padj=-1.0)
axis(side=2, at=seq(1.5, 5.5, by=0.5), labels=seq(1.5, 5.5, by=0.5), padj=1.0)
abline(lm(dataset$maintainability ~ dataset$logCEM))
label <- expression(paste(rho, " = -0.34"))
text(10.8, 2.8, labels=label)
nrPoints <- nrow(dataset[!is.na(dataset$logCEM) & !is.na(dataset$maintainability),])
text(10.8, 1.5, labels=paste("n = ",nrPoints,sep=""), font=3)
title(xlab="log(CEM)", line=1.2)
title(ylab="Maintainability", line=1.5)

dev.off()