#!/usr/bin/env Rscript

# Create different scatterplots for paper
cat("Loading file from hypotheses.r…\n")
load(file=".RData")

png("../paper/scatterMaintNrUnits.png", height=650, width=650, 
	units="px", points=16, antialias="subpixel") 

par(mfrow=c(2,2), mar=c(3,3,0,0), mai=c(.5,.5,0.1,0.1))

dataset <- merge(dataset, statsTable, by.x="fileId", by.y="fileId", all.x=TRUE, all.y=FALSE)

dataset$logCEM <- log(dataset$CEM)
dataset[dataset$logCEM == -Inf & !is.na(dataset$logCEM),]$logCEM <- 0
dataset$logCEM <- (-1 * dataset$logCEM)

dataset$logWRM <- log(dataset$WRM)
dataset[dataset$logWRM == -Inf & !is.na(dataset$logWRM),]$logWRM <- 0
dataset$logWRM <- (-1 * dataset$logWRM)

dataset$logPNM <- log(dataset$PNM)
dataset[dataset$logPNM == -Inf & !is.na(dataset$logPNM),]$logPNM <- 0
dataset$logPNM <- (-1 * dataset$logPNM)

cat("Creating a scatterplot for log(WRM) vs log(CEM)…\n")
plot(abs(dataset$logWRM), -1 * dataset$logCEM,cex=0.3, 
	col=rgb(160,160,160,maxColorValue=255),pch=16,
	main="",
	xlim=c(1, 11), ylim=c(-15, -1),
	xaxt="n", yaxt="n",
	xlab="", ylab="")
axis(side=1, at=seq(1, 11, by=1), labels=seq(1, 11, by=1), padj=-1.0)
axis(side=2, at=seq(-15, -1, by=2), labels=seq(-15, -1, by=2), padj=1.0)
# -8 and -1* to get the line in the middle of the picture, otherwise it doesn't show properly
abline(lm(-8 + -1*dataset$logCEM ~ dataset$logWRM))
label <- expression(paste(rho, " = 0.53"))
text(8.5, -3.5, labels=label)
nrPoints <- nrow(dataset[!is.na(dataset$logWRM) & !is.na(dataset$PNM),])
text(10, -15, labels=paste("n = ",nrPoints,sep=""), font=3)
title(xlab="log(WRM)", line=1.2)
title(ylab="log(CEM)", line=1.3)

cat("Creating a scatterplot for log(nM) vs log(WRM)…\n")
plot(log(dataset$nrUnits), log(dataset$WRM), cex=0.3,
	col=rgb(100,100,100,50,maxColorValue=255), pch=16,
	main="",
	xaxt="n", yaxt="n",
	xlim=c(0, 10), ylim=c(1, 12),
	xlab="", ylab="")
axis(side=1, at=seq(0, 10, by=1), labels=seq(0, 10, by=1), padj=-1.0)
axis(side=2, at=seq(1, 12, by=2), labels=seq(1, 12, by=2), padj=1.0)
abline(lm(3 + -1*dataset$logWRM ~ log(dataset$nrUnits)))
label <- expression(paste(rho, " = 0.36"))
text(8.7, 4.5, labels=label)
nrPoints <- nrow(dataset[!is.na(dataset$logWRM) & !is.na(log(dataset$nrUnits)),])
text(9, 1, labels=paste("n = ",nrPoints,sep=""), font=3)
title(xlab="log(nM)", line=1.2)
title(ylab="log(WRM)", line=1.3)

cat("Creating a scatterplot for nM vs outgoingIsolation…\n")
plot(log(dataset$nrUnits), dataset$outgoingIsolation, cex=0.3, 
	col=rgb(160,160,160,50,maxColorValue=255), pch=16, 
	main="", 
	xaxt="n", yaxt="n",
	xlim=c(0, 10), ylim=c(0.05, 0.96),
	xlab="", ylab="")
axis(side=1, at=seq(0, 10, by=1), labels=seq(0, 10, by=1), padj=-1.0)
axis(side=2, at=c(0.02, seq(0.2, 0.8, by=0.2), 0.99), labels=seq(0, 1, by=0.2), padj=1.0)
abline(lm(dataset$outgoingIsolation ~ log(dataset$nrUnits)))
label <- expression(paste(rho, " = 0.47"))
text(9, 0.87, labels=label)
nrPoints <- nrow(dataset[!is.na(dataset$nrUnits) & !is.na(dataset$outgoingIsolation),])
text(9, 0.05, labels=paste("n = ",nrPoints,sep=""), font=3)
title(xlab="log(nM)", line=1.2)
title(ylab="outI", line=1.5)

cat("creating a scatterplot for nrUnits vs incomingIsolation…\n")
plot(log(dataset$nrUnits), dataset$incomingIsolation, cex=0.3, 
	col=rgb(110,110,110,50,maxColorValue=255), pch=16, 
	main="",
	xaxt="n", yaxt="n",
	xlim=c(0, 10), ylim=c(0.05, 0.96),
	xlab="", ylab="")
axis(side=1, at=seq(0, 10, by=1), labels=seq(0, 10, by=1), padj=-1.0)
axis(side=2, at=c(0.02, seq(0.2, 0.8, by=0.2), 0.99), labels=seq(0, 1, by=0.2), padj=1.0)
abline(lm(dataset$incomingIsolation ~ log(dataset$nrUnits)))
label <- expression(paste(rho, " = -0.11"))
nrPoints <- nrow(dataset[!is.na(dataset$nrUnits) & !is.na(dataset$incomingIsolation),])
text(9.1, 0.05, labels=paste("n = ",nrPoints,sep=""), font=3)
text(9, 0.5, labels=label)
title(xlab="log(nM)", line=1.2)
title(ylab="inI", line=1.5)

dev.off()