#!/usr/bin/env Rscript

# Stata commands:
# predict u01 u02, reffects
# predict u01se u02se, reses
# egen pickone = tag(toLib)
# sort u01
# generate u01rank = sum(pickone)
# sort toLib
# outsheet toLib u01 u01se u01rank using "////serrbar.csv", delimiter(";") replace

serrbar <- read.csv(file="serrbar3.csv", sep=";")
serrbar$toLib <- as.numeric(as.character(serrbar$toLib))
serrbar$est <- as.numeric(as.character(serrbar$est))
serrbar$se <- as.numeric(as.character(serrbar$se))
serrbar$rank <- as.numeric(as.character(serrbar$rank))

serrbar <- serrbar[order(serrbar$rank),]
serrbar <- serrbar[!duplicated(serrbar$toLib),]
serrbar$rank <- c(1:nrow(serrbar))

pdf("../paper/toLibEstimates.pdf")
par(mar=c(3,3,0,0), mai=c(0.3,.5,0.1,0.1))

plot(serrbar$est ~ serrbar$rank, xlim=c(1, nrow(serrbar)), ylim=c(-1.7, 1.7), col="white", xaxt="n", yaxt="n", xlab="", ylab="")
axis(side=2, at=seq(-2.0, 2.0, by=0.5), labels=seq(-2.0, 2.0, by=0.5), padj=1)
title(xlab="Libraries (n = 3374)",line=0.2)
title(ylab="Estimated additive effect on WRM",line=1.4)

segments(serrbar$rank, serrbar$est - (1.96 * serrbar$se), serrbar$rank, serrbar$est - (1.96 * serrbar$se), lty=1, lwd=0.25, col="lightgrey") # Lower band
segments(serrbar$rank, serrbar$est + (1.96 * serrbar$se), serrbar$rank, serrbar$est + (1.96 * serrbar$se), lty=1, lwd=0.25, col="lightgrey") # Upper band

# Plot higher significant points in separate style
highersignif <- serrbar[serrbar$est - (1.96 * serrbar$se) > 0,]
segments(highersignif$rank, highersignif$est - (1.96 * highersignif$se), highersignif$rank, highersignif$est + (1.96 * highersignif$se), lty=3, lwd=2, col="darkgrey")
segments(highersignif$rank - 20, highersignif$est - (1.96 * highersignif$se), highersignif$rank + 20, highersignif$est - (1.96 * highersignif$se), lty=1, lwd=2, col="darkgrey")
segments(highersignif$rank - 20, highersignif$est + (1.96 * highersignif$se), highersignif$rank + 20, highersignif$est + (1.96 * highersignif$se), lty=1, lwd=2, col="darkgrey")
points(highersignif$est ~ highersignif$rank, pch=4, xlim=c(1, nrow(highersignif)), ylim=c(-2, 2), xaxt="n", xlab="", cex=1, col="black")

# Plot lower significant points in separate style
lowersignif <-  serrbar[serrbar$est + (1.96 * serrbar$se) < 0,]
segments(lowersignif$rank, lowersignif$est - (1.96 * lowersignif$se), lowersignif$rank, lowersignif$est + (1.96 * lowersignif$se), lty=3, lwd=2, col="darkgrey")
segments(lowersignif$rank - 20, lowersignif$est - (1.96 * lowersignif$se), lowersignif$rank + 20, lowersignif$est - (1.96 * lowersignif$se), lty=1, lwd=2, col="darkgrey")
segments(lowersignif$rank - 20, lowersignif$est + (1.96 * lowersignif$se), lowersignif$rank + 20, lowersignif$est + (1.96 * lowersignif$se), lty=1, lwd=2, col="darkgrey")
points(lowersignif$est ~ lowersignif$rank, pch=4, xlim=c(1, nrow(lowersignif)), ylim=c(-2, 2), xaxt="n", xlab="", cex=1, col="black")

lines(serrbar$est ~ serrbar$rank, cex=0.5, pch=20, col="black")
abline(h=0) # Add horizontal line at y = 0

text(x=125,y=-1.75,"Bottom 3 libs:\nparanamer-generator\nnarcissus-core\nrdfbean-maven-plugin", adj=c(0,0), cex=1)
text(2250,y=1.3,"Top 3 libs:\nobex-maven-plugin\nhchelpers-assembly\nspring-extensions", adj=c(0,0), cex=1)

dev.off()