#!/bin/bash

if [ $# -lt 2 ]; then
	echo "Usage: $0 <sourceJar> <outputDir>"
	exit 1
fi

sourceJar=$1
outputDir=$2
scriptsDir=$(pwd)

if [ -z $outputDir ]; then
	echo "No outputDir given. Exiting now."
	exit 1
fi

if [ ! -d $outputDir ]; then
	mkdir -p $outputDir
fi

echo "Unzipping '$sourceJar' into '$outputDir'"

rm -rf $outputDir
mkdir -p $outputDir

if [ -d $outputDir ]; then
	echo "Directory $outputDir succesfully created"
else
	echo "Problem in creating $outputDir. Exiting now."
	exit 1
fi

newFilename=$outputDir/$(basename $sourceJar)
cp $sourceJar $newFilename
cd $outputDir

if [ -e $newFilename ]; then
	echo "File '$newFilename' successfully created."
else
	echo "Problem in creating $newFilename. Exiting now."
	exit 1
fi

echo "Checking if file is corrupt"
jar -tf $(basename $sourceJar) 1>/dev/null 2>/dev/null
exitCode=$?
if [ $exitCode -ne 0 ]; then
	echo "Corrupt jar file found. Continuing"
	exit 1
fi

echo "Running jar -xf in '$(pwd)' on file $(basename $sourceJar)"
jar -xf $(basename $sourceJar) 1>/dev/null
exitCode=$?
echo "jar -xf finished with exit code $?"

nrJavaFiles=$(echo $(find $(pwd) -type f -name "*.java" | wc -l))
nrLinesOfCode=$(echo $(find $(pwd) -type f -name "*.java" | xargs cat | wc -l))

echo "Found $nrJavaFiles java files with a total of $nrLinesOfCode LOC."
echo "Contents of temporary directory '$(pwd)':"
ls -lh

if [ $nrJavaFiles -eq 0 ]; then
	echo "No java files found. Exiting now."
	echo ""
	exit 1
fi

cd $scriptsDir
exit $exitCode