#!/bin/bash

if [ $# -lt 5 ]; then
	echo "Usage: $0 <scriptsDir> <jarDir> <outputDir> <programDir> <shouldDecompile>" >&2
	exit 1
fi

# The directory where the scripts are located
scriptsDir=$(cd $1; pwd)
# The directory where the jar to unzip is located
jarDir=$(cd $2; pwd)
# The directory where the output files should be located
outputDir=$(cd $3; pwd)
# The directory of external programs to call in decompile.sh
programDir=$(cd $4; pwd)
# Whether sources should be decompiled or not
shouldDecompile=$5

cd $jarDir

binaryJar=$(find $(pwd) -type f -name "*.jar" -maxdepth 1 | grep -v ".*-sources.jar$" | grep -v ".*-javadoc.jar$")
sourceJar=$(find $(pwd) -type f -name "*.jar" -maxdepth 1 | grep ".*-sources.jar$")

echo "   binaryJar: '$binaryJar'"
echo "   sourceJar: '$sourceJar'"

if [ $shouldDecompile -eq 1 ]; then
	echo "Decompiling source '$binaryJar' to outputDir '$outputDir'"
	$scriptsDir/decompile.sh $binaryJar $outputDir $programDir
else
	echo "Unzipping source '$sourceJar' to outputDir '$outputDir'"
	$scriptsDir/unzipsources.sh $sourceJar $outputDir
fi

exit $?